import {
  Body1,
  Button,
  Caption1,
  Card,
  CardFooter,
  CardHeader,
  CardPreview,
  Spinner,
  Tooltip,
} from "@fluentui/react-components";
import {
  DesktopRegular,
  FluentIcon,
  Play16Regular,
  PowerRegular,
  StopRegular,
} from "@fluentui/react-icons";
import React from "react";
import { VMApi, VMInfo, VMState } from "../api/VMApi";
import { AsyncWidget } from "./AsyncWidget";
import { SectionContainer } from "./SectionContainer";
import { useToast } from "../hooks/providers/ToastProvider";

export function VirtualMachinesWidget(): React.ReactElement {
  const [list, setList] = React.useState<VMInfo[] | undefined>();
  const load = async () => {
    setList(await VMApi.GetList());
  };

  return (
    <SectionContainer>
      <AsyncWidget
        loadKey={1}
        load={load}
        loadingMessage="Loading the list virtual machines..."
        errMsg="Failed to load the list of virtual machines!"
        build={() => <VirtualMachinesWidgetInner list={list!} />}
      />
    </SectionContainer>
  );
}

function VirtualMachinesWidgetInner(p: { list: VMInfo[] }): React.ReactElement {
  return (
    <div
      style={{
        display: "flex",
        flexDirection: "row",
        flexWrap: "wrap",
        justifyContent: "center",
      }}
    >
      {p.list.map((v, n) => (
        <VMWidget key={n} vm={v} />
      ))}{" "}
    </div>
  );
}

function VMWidget(p: { vm: VMInfo }): React.ReactElement {
  const toast = useToast();

  const [state, setState] = React.useState<VMState | undefined>();

  const load = async () => {
    const newState = await VMApi.State(p.vm);
    if (state !== newState) setState(newState);
  };

  React.useEffect(() => {
    const interval = setInterval(async () => {
      try {
        if (p.vm.can_get_state) await load();
      } catch (e) {
        console.error(e);
        toast("Error", `Failed to refresh ${p.vm.name} status!`, "error");
      }
    }, 1000);
    return () => clearInterval(interval);
  });

  return (
    <Card
      style={{
        width: "400px",
        maxWidth: "49%",
        height: "220px",
        margin: "10px",
        display: "flex",
        flexDirection: "column",
      }}
    >
      <CardPreview>TODO preview</CardPreview>

      <CardHeader
        image={<DesktopRegular fontSize={32} />}
        header={
          <Body1>
            <b>{p.vm.name}</b>
          </Body1>
        }
        description={
          <Caption1>
            {p.vm.can_get_state ? state ?? "..." : "Unavailable"}
          </Caption1>
        }
      />

      <p style={{ flex: 1 }}>{p.vm.description}</p>

      <CardFooter>
        <VMAction
          {...p}
          primary
          label="Start"
          icon={<Play16Regular />}
          enabled={p.vm.can_start}
          currState={state}
          possibleStates={["Shutdown", "Shutoff", "Crashed"]}
          onClick={VMApi.StartVM}
        />
        <VMAction
          {...p}
          label="Shutdown"
          icon={<PowerRegular />}
          enabled={p.vm.can_shutdown}
          currState={state}
          possibleStates={["Running"]}
          onClick={VMApi.ShutdownVM}
        />
        <VMAction
          {...p}
          label="Kill"
          icon={<StopRegular />}
          enabled={p.vm.can_kill}
          currState={state}
          possibleStates={[
            "Running",
            "Paused",
            "PowerManagementSuspended",
            "Blocked",
          ]}
          onClick={VMApi.KillVM}
        />
      </CardFooter>
    </Card>
  );
}

function VMAction(p: {
  vm: VMInfo;
  label: string;
  primary?: boolean;
  icon: React.ReactElement;
  enabled: boolean;
  currState?: VMState;
  possibleStates: VMState[];
  onClick: (vm: VMInfo) => Promise<void>;
}): React.ReactElement {
  const toast = useToast();
  const [loading, setLoading] = React.useState(false);

  const onClick = async () => {
    try {
      setLoading(true);

      await p.onClick(p.vm);

      toast(p.label, `Action successfully executed!`, "success");
    } catch (e) {
      console.error(e);
      toast(p.label, `Failed to perform action: ${e}`, "error");
    } finally {
      setLoading(false);
    }
  };

  if (!p.currState || !p.possibleStates.includes(p.currState)) {
    return <></>;
  }

  if (!p.enabled)
    return (
      <Tooltip content={"Unavailable"} relationship="label">
        <Button
          appearance={p.primary ? "primary" : undefined}
          icon={p.icon}
          disabled
        >
          {p.label}
        </Button>
      </Tooltip>
    );

  return (
    <Button
      appearance={p.primary ? "primary" : undefined}
      icon={loading ? <Spinner size="tiny" /> : p.icon}
      onClick={onClick}
    >
      {p.label}
    </Button>
  );
}
